/* Copyright (C)  Walter L. Peacock 1984-1988.  All rights reserved */

/* THIS PROGRAM BELONGS TO WALTER L. PEACOCK.  IT IS CONSIDERED A TRADE */
/* SECRET AND IS NOT TO BE DIVULGED OR USED BY PARTIES WHO HAVE NOT    */
/* RECEIVED WRITTEN AUTHORIZATION FROM THE OWNER.                */

/*---------------------------------------------------------------------------- 
 * The btcinit function is invoked to initialize the btcommo structure for a 
 * btree.  Btree parameters are passed to this function as arguments instead of
 * being read from btparms.btr.  
 * This function can be called by your applications to create btrees on the fly
 * without the need to read the parameter info in btparms.btr, however, your
 * programs will need to remember the parameters passed the next time that the
 * btree is to be accessed.
 * 
 * This routine is also invoked by btrinit() which supplies its parameters  
 * from btparms.btr.  The btcommo members are initialized as follows:
 *
 *   btoptype  is set to GETREC as a default,
 *   btroot    is set to 0L,
 *   btidxlen  is set to the value passed.
 *   btidxnme  is set to the value passed.
 *   btcells   is set to the value calculated by CBTREE formula.
 *   btdtalen  is set to the value passed.
 *   btkeylen  is set to the value passed.
 *   btkey     is set to an allocated memory block of btkeylen + 1 bytes,
 *   btmax     is set to the value passed (must be > 0).
 *   btrecnum  is set to an allocated memory block of btmax + 1 longs,
 *      
 * The character string idxname passed to btcinit() identifies the btree name.
 * idxfname identifies the index file name that contains the btree.
 * A valid name contains 1 to 8 alphanumeric characters.
 *
 * if( (btcinit(&btnme, "patient.idx",nam_btnme,512,22,128,128) ) == ERR)
 * {
 *    ckerror( CK_BTRIN, "bld2var: btnme");
 *    goto b2v_0exit;
 * }
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "cbtree.h"
#include "btfio.h"

#if CI | DC
#else
#include <ctype.h>
#endif

/* 03/12/88 btcinit.c   */

int btcinit(btcp, idxfname, idxname, idxsize, keysize, datasize, btmaxsz)
BTC *btcp;			/* btcommo structure tag. */
char *idxfname;		/* index file name. */
char *idxname;		/* index btree name. */
unsigned idxsize;	/* index block size. */
unsigned keysize;	/* index maximum key size. */
unsigned datasize;	/* data file fixed length record size, 0 for varible. */
unsigned btmaxsz;	/* # of cells to calloc for block calls: GETALL, GETKEYS. */
{
   extern long findroot();
   extern int  close();
   extern void free();
   extern char *calloc();
   BT_BASE btbase;
   BT_INFO btinfo;
   int fd;
   register int i;

   if ((btcp->dtaname = calloc(30, SZCHAR)) == NULL)
      ckerror( - CK_NOMEM, "79");

   btcp->btdtalen = datasize;
   btcp->btvarlen = (btcp->btdtalen == 0);
   btcp->btmulti = 0; 
   btcp->btcrash = 0; 

   if ((fd = bt_open(idxfname, O_RDONLY)) == ERR)
      btcp->findroot = -1L;
   else
   {
      if ((btcp->findroot = findroot(fd, idxname)) > 0L)
         getinfo(&btinfo, fd, btcp->findroot);  /* get embedded parameters */
      else if (btcp->findroot == 0L)
         getbase(&btbase, fd);
      close (fd);
   }

   btcp->btroot = 0L;
   btcp->idxname = calloc(MAXFNAME + 1, SZCHAR);   /* cc220 */
   if (btcp->idxname == NULL)
      ckerror( - CK_NOMEM, "85");
   strcpy(btcp->idxname, idxfname);    /* index file name */
   btcp->btidxlen = idxsize;
   btcp->btkeylen = keysize;

   /* btcells calculation is from btsetup.c.  */
   btcp->btcells = (idxsize - 2 * sizeof(long) - 2 * sizeof(uword) - 2) /
         (sizeof(long) + keysize + SZCHAR);

   if ((btcp->findroot > 0L  &&     /* if we have good INFO */
       (btcp->btidxlen != btinfo.bi_blksz ||
         btcp->btkeylen > btinfo.bi_kylen ||
         btcp->btcells != btinfo.bi_cells ||
         btcp->btvarlen != btinfo.bi_varlen )) ||
       (btcp->findroot == 0L &&
        (btcp->btidxlen != btbase.bb_blksz ||
         btcp->btvarlen != btbase.bb_varlen)))
      ckerror(- CK_INDX, "131");

   btcp->btname = calloc(8 + 1, SZCHAR);    /* cc220 */
   if (btcp->btname == NULL)
      ckerror( - CK_NOMEM, "84");
   strcpy(btcp->btname, idxname);    /* btree name */

   btcp->btoptype = GETREC; 	  /* Default */
   if ((btcp->btkey = calloc(btcp->btkeylen + 1, SZCHAR)) == NULL)
      ckerror( - CK_NOMEM, "27");
   btcp->btloc  = 0L;

   btcp->btoptyp2 = GETREC;   /* Default */
   if ((btcp->btkey2 = calloc(btcp->btkeylen + 1, SZCHAR)) == NULL)
      ckerror( - CK_NOMEM, "28");
   btcp->btloc2  = 0L;

   if(btmaxsz < 1)
	btmaxsz = 1;
   i = btcp->btmax = btmaxsz;
   btcp->btrecnum = (long *)calloc(i + 1, sizeof(long));
   if (btcp->btrecnum == NULL)
      ckerror( - CK_NOMEM, "80");
   btcp->btrecnm2 = (long *)calloc(i + 1, sizeof(long));
   if (btcp->btrecnm2 == NULL)
      ckerror( - CK_NOMEM, "29");

   btcp->btsvkey = NULL;
   btcp->btsave = 0;

   return(1);
}

void btrterm(btcp)   /* terminate use of given b+tree */
BTC *btcp;
{
   extern void free();

   FREE(btcp->dtaname);
   FREE(btcp->idxname);
   FREE(btcp->btname);
   FREE(btcp->btkey);
   FREE(btcp->btkey2);
   FREE(btcp->btrecnum);
   FREE(btcp->btrecnm2);
}

